Imports System.Runtime.InteropServices 'APIs
'The System.Reflection Namespace Contains Classes And Interfaces That 
'Provide A Managed View Of Loaded Types, Methods, And Fields, With The 
'Ability To Dynamically Create And Invoke Types.
Imports System.Reflection
Public Class Form1

    <DllImport("user32", EntryPoint:="GetCaretBlinkTime", SetLastError:=True, CharSet:=CharSet.Auto)> _
    Private Shared Function GetCaretBlinkTime() As Int32
    End Function 'GetCaretBlinkTime API

    <DllImport("user32", EntryPoint:="SetCaretBlinkTime", SetLastError:=True, CharSet:=CharSet.Auto)> _
    Private Shared Function SetCaretBlinkTime(ByVal wMSeconds As Int32) As Int32
    End Function 'Set Caret Blink Time

    <DllImport("user32", EntryPoint:="ShowCursor", SetLastError:=True, CharSet:=CharSet.Auto)> _
        Private Shared Function ShowCursor(ByVal bShow As Int32) As Int32
    End Function 'Show Cursor

    <StructLayout(LayoutKind.Sequential)> _
    Private Structure RECT
        Public Left As Int32
        Public Top As Int32
        Public Right As Int32
        Public Bottom As Int32
    End Structure 'Rectangular Bounds

    <DllImport("user32", EntryPoint:="LoadCursorFromFile", SetLastError:=True, CharSet:=CharSet.Unicode)> _
    Private Shared Function LoadCursorFromFile(ByVal filename As String) As IntPtr
    End Function

    Private CurrentSpeed As Int32 'Store Current Speed Of Cursor
    Private DefaultTime As Int32 'New Cursor Speed

    Private MouseTrap As RECT 'Rect Object
    Private CurseInt As Integer 'Cursor Counter
    Private CursorArray() As Cursor 'Array To Store Cursors

    ''' <summary>
    ''' Restore Cursor Bounds - Full Screen
    ''' </summary>
    ''' <param name="ThisForm"></param>
    ''' <remarks></remarks>
    Public Sub RestoreCursor(ByRef ThisForm As System.Windows.Forms.Form)
        With MouseTrap
            .Top = 0
            .Left = 0
            .Right = Screen.PrimaryScreen.Bounds.Width
            .Bottom = Screen.PrimaryScreen.Bounds.Height
        End With

        'no need for clip cursor
        System.Windows.Forms.Cursor.Clip = New Rectangle(MouseTrap.Left, MouseTrap.Top, MouseTrap.Right, MouseTrap.Bottom) 'Restore Bounds

    End Sub

    ''New Cursor Pos & Confine To Form
    Private Sub MoveCursor()
        ' set the Current cursor, move the cursor's Position,
        ' and set its clipping rectangle to the form. 
        Me.Cursor = New Cursor(System.Windows.Forms.Cursor.Current.Handle)
        System.Windows.Forms.Cursor.Position = New Point(Me.Left + 50, Me.Top + 50)
        System.Windows.Forms.Cursor.Clip = New Rectangle(Me.Left + 50, Me.Top + 50, Me.Width - 50, Me.Height - 50)
    End Sub

    Private Shared Function CreateCursor(ByVal filename As String) As Cursor
        Dim hCursor As IntPtr 'Handle To The Cursor
        Dim result As Cursor = Nothing 'Returned Cursor

        Try
            hCursor = LoadCursorFromFile(filename) 'Load
            If Not IntPtr.Zero.Equals(hCursor) Then 'If Exists
                result = New Cursor(hCursor) 'Obtain New Cursor
            Else
                'Could Not Create Cursor
                Throw New ApplicationException("Could not create cursor from file " & filename)
            End If
        Catch ex As Exception
            'Log Your Exception
        End Try

        Return result 'Return The Cursor
    End Function

    Private Sub bSpeed_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bSpeed.Click
        CurrentSpeed = SetCaretBlinkTime(DefaultTime) 'Set New Caret Blink Time

        If CurrentSpeed > 0 Then 'Only If > 0
            ' Display the new blink rate.
            hSpeed.Value = DefaultTime 'Set Scrollbar
            tSpeed.Text = Integer.Parse(GetCaretBlinkTime) 'Display Blink Time
            hSpeed.Focus() 'Set Focus
        End If
    End Sub

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        CurrentSpeed = SetCaretBlinkTime(DefaultTime) 'Restore Old Blink Time

    End Sub

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        'Load System Cursors From System Using Reflection
        For Each p As PropertyInfo In GetType(Cursors).GetProperties
            If (p.PropertyType Is GetType(Cursor)) Then
                cboCursors.Items.Add("Cursors." & p.Name)
            End If
        Next

        ReDim CursorArray(cboCursors.Items.Count) 'Redim Array To Length Of cbo Array
        CursorArray(0) = Cursors.AppStarting 'Set Cursors
        CursorArray(1) = Cursors.Arrow
        CursorArray(2) = Cursors.Cross
        CursorArray(3) = Cursors.Default
        CursorArray(4) = Cursors.Hand
        CursorArray(5) = Cursors.Help
        CursorArray(6) = Cursors.HSplit
        CursorArray(7) = Cursors.IBeam
        CursorArray(8) = Cursors.No
        CursorArray(9) = Cursors.NoMove2D
        CursorArray(10) = Cursors.NoMoveHoriz
        CursorArray(11) = Cursors.NoMoveVert
        CursorArray(12) = Cursors.PanEast
        CursorArray(13) = Cursors.PanNE
        CursorArray(14) = Cursors.PanNorth
        CursorArray(15) = Cursors.PanNW
        CursorArray(16) = Cursors.PanSE
        CursorArray(17) = Cursors.PanSouth
        CursorArray(18) = Cursors.PanSW
        CursorArray(19) = Cursors.PanWest
        CursorArray(20) = Cursors.SizeAll
        CursorArray(21) = Cursors.SizeNESW
        CursorArray(22) = Cursors.SizeNS
        CursorArray(23) = Cursors.SizeNWSE
        CursorArray(24) = Cursors.SizeWE
        CursorArray(25) = Cursors.UpArrow
        CursorArray(26) = Cursors.VSplit
        CursorArray(27) = Cursors.WaitCursor

        CurrentSpeed = GetCaretBlinkTime() 'Current / Old Blink Rate

        If CurrentSpeed > 0 Then
            DefaultTime = CurrentSpeed
            hSpeed.Minimum = 10 'Min ScrollBar Value
            hSpeed.Maximum = (1000 + hSpeed.LargeChange - 1) 'Multiply Scrollbar Val With 1000 (Second)
            hSpeed.Value = DefaultTime
            tSpeed.Text = Integer.Parse(DefaultTime)
        End If
    End Sub

    Private Sub hSpeed_Scroll(ByVal sender As Object, ByVal e As System.Windows.Forms.ScrollEventArgs) Handles hSpeed.Scroll
        CurrentSpeed = SetCaretBlinkTime(hSpeed.Value) 'Set New Blink Rate

        If CurrentSpeed > 0 Then

            CurrentSpeed = GetCaretBlinkTime()
            tSpeed.Text = CStr(CurrentSpeed)
        End If

    End Sub

    Private Sub tSpeed_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles tSpeed.TextChanged
        Dim iTextValue As Short
        iTextValue = Short.Parse(tSpeed.Text)

        If iTextValue > 1000 Or iTextValue < 10 Then 'Only Between 10 And 1000
            MessageBox.Show("Enter a number between 10 and 1000.")
            tSpeed.Text = Integer.Parse(hSpeed.Value)
        Else
            hSpeed.Value = iTextValue
        End If
    End Sub

    Private Sub bHideF_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bHideF.Click
        ShowCursor(False) 'Hide Cursor On Form
    End Sub

    Private Sub bShowC_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bShowC.Click
        RestoreCursor(Me) 'Restore Cursor Bounds
        ShowCursor(True) 'Show Cursor
    End Sub

    Private Sub bHideE_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bHideE.Click
        ShowCursor(False) 'Hide Cursor
        MoveCursor() 'Confine Hidden Cursor To Form, Otherwise Still Visible Outside Of Our Form
    End Sub

    Private Sub bCP_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bCP.Click
        System.Windows.Forms.Cursor.Position = New Point(500, 500) 'Set New Position
    End Sub

    Private Sub cboCursors_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles cboCursors.SelectedIndexChanged
        If cboCursors.SelectedIndex > -1 Then
            CurseInt = (cboCursors.SelectedIndex) 'Get Selected Value

        End If
    End Sub

    Private Sub bChangeCF_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bChangeCF.Click
        Me.Cursor = Cursors.AppStarting
        bLoadIC.Cursor = Cursors.AppStarting

        Me.Cursor = CursorArray(CurseInt) 'Set Form's New Cursor
    End Sub

    Private Sub bLoadIC_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bLoadIC.Click
        Me.Cursor = New Cursor("Bullseye.cur") 'Load Cursor From File
    End Sub

    Private Sub btnAnimate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnAnimate.Click

        Me.Cursor = CreateCursor(Application.StartupPath & "\dinosaur.ani")

    End Sub
End Class
